<template>
  <div class="center-slider">
    <div v-show="powType" class="center-slider-main">
      <div id="slider" ref="slider" :style="`opacity: ${temAbleSet ? 1 : 0.01}`" />
      <div
        class="layer"
        :style="{
          width: lottieRadius / 1.5 + 'px',
          height: lottieRadius / 1.5 + 'px',
          'background-position': `0 ${roundBg * 100}%`
        }"
      >
        <div class="rotate -one"></div>
        <div class="rotate -two"></div>
      </div>
      <article>
        <!-- 显示模式/风档 -->
        <gree-block v-if="modText.length && fanName.length">
          <div class="left">
            <span v-text="modText" />
          </div>
          <div class="bar" />
          <div class="right">
            <span v-text="fanName" />
          </div>
        </gree-block>
        <gree-block v-else>
          <span v-if="modText.length" v-text="modText" />
          <span v-if="fanName.length" v-text="fanName" />
        </gree-block>
        <!-- 温度调节 -->
        <div v-if="circleControlAble" class="tem-value" :class="{ 'deci-tem': temStep < 1 }">
          <gree-animated-number class="tem-value-int" :value="circleValInt" :precision="0" :duration="200" :transition="false" />
          <!-- <gree-animated-number class="tem-value-F" :value="circleValIntF" :precision="0" :duration="200" :transition="false" v-else /> -->
          <gree-animated-number v-if="temStep < 1" class="tem-value-dec" :value="circleValDec * 10" :precision="0" :duration="200" :transition="false" />
          <div
            :class="{ 'unit-tem': temSetUnit === 'SetTem', 'unit-humi': temSetUnit === 'SetCoolHumi', 'unit-fah': temSetUnit === 'SetTemF' }"
            v-text="circleValUnit"
          />
        </div>
        <!-- 显示插槽2 -->
        <h3
          v-if="showSolt2"
          class="slider-slot-2"
          :class="{ GoOut: this.GoOut || this.StHt, LowDeHumi: this.LowDeHumi, MobileMod: isMobileMod }"
          v-text="imshowSlot2"
        />
        <!-- 显示插槽3 -->
        <h3 v-if="showSolt3" class="slider-slot-3" :class="{ GoOut: this.GoOut, LowDeHumi: this.LowDeHumi }" v-text="specialVal" />
        <!-- 显示插槽1 -->
        <div v-if="imshowSlot1" class="slider-slot-1">
          <span v-text="imshowSlot1" />
        </div>
      </article>
    </div>
    <!-- 加减按钮 -->
    <div class="center-slider-btn-group" :class="{ 'set-gray': !circleControlAble && !this.GoOut && !this.LowDeHumi }" v-if="powType">
      <div
        class="center-slider-btn-group-minus"
        :class="{ disable: !minusBottonAble }"
        @click="setTemByStep(-1)"
        v-longclick="() => setTemByLongClick(-1)"
        @touchend="onTouchend()"
      />
      <div
        class="center-slider-btn-group-plus"
        :class="{ disable: !plusBottonAble }"
        @click="setTemByStep(1)"
        v-longclick="() => setTemByLongClick(1)"
        @touchend="onTouchend()"
      />
    </div>
    <div class="pow-off" v-else :style="{ width: svgRadius + 63.5 + 'px', height: svgRadius + 63.5 + 'px' }">
      <h3 v-text="$language('home.powerOff')" />
    </div>
  </div>
</template>

<script>
import { Block, AnimatedNumber, Overlay } from 'gree-ui';
import { mapActions, mapGetters, mapState, mapMutations } from 'vuex';
import { types } from '@/store/types';
import temUnChange from '@/mixins/tem';
import Vue from 'vue';
import debounce from 'lodash/debounce';
import { longClickDirective } from 'vue-long-click';
const longClickInstance = longClickDirective({ delay: 200, interval: 150 });
Vue.directive('longclick', longClickInstance);

export default {
  props: {
    roundBg: {
      type: Number,
      default: 0
    }
  },
  components: {
    [Block.name]: Block,
    [AnimatedNumber.name]: AnimatedNumber,
    [Overlay.name]: Overlay
  },
  mixins: [temUnChange],
  data() {
    return {
      isShowToolTip: false,
      minusBottonAble: true, // 减按钮是否可点击
      plusBottonAble: true, // 加按钮是否可点击
      svgRadius: 0,
      isClick: false,
      lottieRadius: 0,
      circleVal: 26,
      temChange: false, // 轮询回来的温度改变flag
      fanName: '',
      circleObj: '',
      timer: null, // 节流函数用
      sliderValueMap: {} // 节流函数用
    };
  },
  created() {
    this.svgRadius = window.screen.width * 0.8;
    this.lottieRadius = this.svgRadius * 1.8;
  },
  computed: {
    ...mapState('control', {
      status: state => state.status,
      GoOut: state => state.dataObject.GoOut,
      LowDeHumi: state => state.dataObject.LowDeHumi,
      HumiEnable: state => state.dataObject.HumiEnable,
      SetCoolHumiMin: state => state.dataObject.SetCoolHumiMin,
      HumiSvStTemMin: state => state.dataObject.HumiSvStTemMin,
      Mod: state => state.dataObject.Mod,
      AutoSetTemEn: state => state.dataObject.AutoSetTemEn,
      TemUn: state => state.dataObject.TemUn || 0,
      SetTem: state => state.dataObject.SetTem,
      StHt: state => state.dataObject.StHt,
      TemRec: state => state.dataObject.TemRec || 0,
      dataObject: state => state.dataObject,
      isLongClick: state => state.status.isLongClick,
      Add01: state => state.dataObject['Add0.1'] || 0,
      Add05: state => state.dataObject['Add0.5'] || 0
    }),
    ...mapGetters([
      'inputMap',
      'modCurrentStatusName',
      'temSetVal',
      'temSetJson',
      'temSetUnit',
      'temMinVal',
      'temMaxVal',
      'temStep',
      'temAbleSet',
      'fanIdentifier',
      'imshowSlot1',
      'imshowSlot2',
      'powType',
      'modTextKey',
      'modDefine'
    ]),
    ...mapGetters('machine', ['funcDefineMap', 'statusMap']),
    modText() {
      return this.$language(this.modTextKey);
    },
    circleControlAble() {
      return !this.imshowSlot2;
    },
    isMobileMod() {
      return this.modDefine[0].identifier === 'MobileMod';
    },
    hasStHt() {
      return Boolean(this.funcDefineMap.StHt);
    },
    showSolt2() {
      if (this.isMobileMod) {
        return this.imshowSlot2;
      } else {
        return (!this.TemUn && (this.GoOut || this.LowDeHumi || this.Mod === 8)) || (this.Mod === 0 && !this.AutoSetTemEn) || (this.hasStHt && this.StHt);
      }
    },
    showSolt3() {
      return this.TemUn && (this.GoOut || this.LowDeHumi);
    },
    circleValInt() {
      const modArr = [5, 6, 7];
      if (modArr.includes(this.Mod) && this.SetTem === 16 && this.TemUn) {
        this.setDataObject({ SetTem: 16 });
      }
      return Math.floor(this.circleVal);
    },
    circleValDec() {
      return this.circleVal % 1;
    },
    circleValUnit() {
      const map = { SetTem: '℃', SetCoolHumi: '%', SetTemF: '℉', undefined: '' };
      const result = map[this.temSetUnit];
      return result;
    },
    specialVal() {
      if (this.GoOut) {
        return '46';
      } else if (this.LowDeHumi) {
        return '54';
      } else return false;
    }
  },
  mounted() {
    this.circleVal = this.temSetVal;
    // init circular slider
    this.circleObj = $('#slider')
      .roundSlider({
        animation: false,
        showTooltip: false,
        min: 16,
        max: 30,
        step: 0.1,
        value: this.circleVal,
        radius: this.svgRadius / 2 + 5,
        width: 5,
        handleSize: '+38',
        keyboardAction: false,
        startAngle: -45,
        endAngle: '+270',
        sliderType: 'min-range',
        circleShape: 'full',
        handleShape: 'dot',
        svgMode: true,
        lineCap: 'round',
        borderWidth: 0,
        borderColor: 'rgb(240, 240, 240)',
        pathColor: 'rgba(255, 255, 255, 0.3)',
        rangeColor: '#ffffff',
        start: () => {
          this.temChange = false;
          this.isShowToolTip = true;
          this.getDotRect();
          // this.addBubblesStyle();
          // $('span.rs-block').css('padding', '2.7px');
        },
        stop: () => {
          this.isShowToolTip = false;
          this.addBubblesStyle();
          // $('span.rs-block').css('padding', '2px');
        },
        drag: e => {
          this.getDotRect();
          this.addBubblesStyle();
          const base = 1 / this.temStep;
          this.circleVal = Math.round(e.value * base) / base;
        },
        beforeValueChange: e => {
          if (Math.abs(e.preValue - e.value) >= (this.temMaxVal - this.temMinVal) / 3 && !this.temChange) return false;
          const base = 1 / this.temStep;
          this.temChange = false;
          // e.value !== this.SetTem && e.value == this.SetTem;
          this.circleVal = Math.round(e.value * base) / base;
        },
        change: e => {
          const base = 1 / this.temStep;
          this.circleVal = Math.round(e.value * base) / base;
          this.temSetMethod(this.circleVal);
          this.throttle(this.circleVal, 'value');
        },
        create: function() {
          this.control
            .find('.rs-handle')
            .addClass('rs-transition')
            .eq(0)
            .rsRotate(-this._handle1.angle);
        }
      })
      .on('change drag', function(event) {
        $(event.handle.element).rsRotate(-event.handle.angle);
      });
  },
  activated() {
    this.setSliderArea();
  },
  watch: {
    statusMap: {
      handler(newVal) {
        if (!newVal) return;
        const fanStateName = this.getStateNameById(this.fanIdentifier);
        if (fanStateName) {
          this.fanName = this.$language(`fan.${fanStateName}`);
        }
      },
      immediate: true,
      deep: true
    },
    // 圆环数值
    temSetVal: {
      handler(newVal) {
        if (this.TemUn) {
          //华氏度调温时整机传回来的温度数值仍是摄氏度的值，所以需要插件转换为华氏度对应值再更新圆环显示
          if ((this.Mod !== 2 || !this.HumiEnable) && newVal < 60) {
            console.log('2-----------------xxxxxxxxxxxxxxxxxx', Math.floor(newVal), this.tempC2F(newVal, this.TemRec));
            this.throttle(this.tempC2F(newVal, this.TemRec), 'value');
          } else {
            this.throttle(Math.floor(newVal), 'value');
          }
          if (this.Mod === 2 && this.HumiEnable) {
            newVal === this.temMaxVal ? (this.plusBottonAble = false) : (this.plusBottonAble = true);
            newVal === this.temMinVal ? (this.minusBottonAble = false) : (this.minusBottonAble = true);
          } else if (newVal < 61) {
            this.tempC2F(newVal, this.TemRec) === this.temMaxVal ? (this.plusBottonAble = false) : (this.plusBottonAble = true);
            this.tempC2F(newVal, this.TemRec) === this.temMinVal ? (this.minusBottonAble = false) : (this.minusBottonAble = true);
          } else {
            newVal === this.temMaxVal ? (this.plusBottonAble = false) : (this.plusBottonAble = true);
            newVal === this.temMinVal ? (this.minusBottonAble = false) : (this.minusBottonAble = true);
          }
        }
        if (!this.TemUn) {
          newVal === this.temMaxVal ? (this.plusBottonAble = false) : (this.plusBottonAble = true);
          newVal === this.temMinVal ? (this.minusBottonAble = false) : (this.minusBottonAble = true);
          this.throttle(newVal, 'value');
        }
        this.temChange = true;
        // if (!this.TemUn) {
        // }
        // else if (this.TemUn) {
        // }
      },
      immediate: true,
      deep: true
    },
    //除湿模式百分比调节标志位
    HumiEnable: {
      handler(newVal) {
        //除湿模式且百分比调节时，节能下限始终保持显示45
        if (newVal && this.modCurrentStatusName === 'status_3') {
          this.throttle(this.SetCoolHumiMin, 'min');
        }
        if (!newVal && this.modCurrentStatusName === 'status_3') {
          //除湿模式且温度调节时，节能下限显示上一次设置的温度下限值
          this.throttle(this.HumiSvStTemMin, 'min');
        }
      }
    },
    Mod: {
      handler(newVal) {
        //在其他模式开启节能后切换到除湿模式，如果是百分比调节，就把圆环下限设为45
        if (newVal === 2 && this.HumiEnable) {
          this.throttle(this.SetCoolHumiMin, 'min');
        }
      }
    },
    // 圆环最小值
    temMinVal: {
      handler(newVal) {
        if (this.HumiEnable && this.modCurrentStatusName === 'status_3') {
          this.throttle(this.SetCoolHumiMin, 'min');
        } else {
          this.throttle(newVal, 'min');
        }
      },
      immediate: true
    },
    // 圆环最大值
    temMaxVal: {
      handler(newVal) {
        this.throttle(newVal, 'max');
      },
      immediate: true
    },
    circleVal: {
      handler(newVal) {
        window.$bus.$emit('updataVal', newVal);
      }
    },
    circleValInt: {
      handler() {
        this.outPutCircleVal();
      },
      immediate: true
    },
    circleValDec: {
      handler() {
        this.outPutCircleVal();
      },
      immediate: true
    },
    circleValUnit: {
      handler() {
        this.outPutCircleVal();
      },
      immediate: true
    },
    circleControlAble(newVal) {
      // 防止圆环显示NaN
      if (newVal && Number.isNaN(this.circleVal)) {
        this.circleVal = this.temSetVal;
      }
    }
  },
  methods: {
    ...mapActions({
      sendData: types.SEND_DATA,
      sendCtrl: types.SEND_CTRL
    }),
    ...mapMutations('control', {
      setDataObject: 'SET_DATA_OBJECT',
      setStatus: 'SET_STATUS'
    }),
    ...mapMutations({
      setCheckObject: types.SET_CHECK_OBJECT
    }),
    // 根据间隔设置温度
    setTemByStep: debounce(function(type) {
      if (this.circleControlAble || this.GoOut || this.LowDeHumi) {
        const step = this.temStep;
        this.temSetMethod(this.circleVal + step * type);
      }
    }, 200),
    // 温度设置方法
    temSetMethod(value) {
      if (value > this.temMaxVal || value < this.temMinVal) return;
      // value === this.temSetVal || this.sendData({ [this.temSetJson]: value });
      if (value !== this.temSetVal || !this.isLongClick) {
        if (this.TemUn) {
          if (this.Mod === 2) {
            // 除湿模式华氏度单独处理
            if (!this.HumiEnable) {
              this.sendData({
                [this.temSetJson]: this.tempF2C(value)[1],
                GoOut: 0,
                LowDeHumi: 0,
                TemRec: this.tempF2C(value)[3]
              });
              this.setDataObject({ 'Add0.1': 0 });
            } else {
              this.sendData({
                [this.temSetJson]: value,
                GoOut: 0,
                LowDeHumi: 0
              });
              this.setDataObject({ 'Add0.1': 0 });
            }
          } else {
            console.log(1123456789, value);
            this.sendData({
              [this.temSetJson]: this.tempF2C(value)[1],
              GoOut: 0,
              LowDeHumi: 0,
              TemRec: this.tempF2C(value)[3]
            });
            this.setDataObject({ 'Add0.1': 0, 'Add0.5': 0 });
          }
        } else {
          this.sendData({
            [this.temSetJson]: value,
            GoOut: 0,
            LowDeHumi: 0
          });
        }
      }
    },
    // 长按后放手时发送温度指令
    onTouchend() {
      if (this.circleControlAble || this.GoOut || this.LowDeHumi) {
        this.setStatus({ isLongClick: false });
        this.temSetMethod(this.circleVal);
      }
    },
    setTemByStepLoCilck(type) {
      if (this.circleControlAble || this.GoOut || this.LowDeHumi) {
        const step = this.temStep;
        this.temSetMethod(this.circleVal + step * type);
      }
    },
    // 长按设置温度
    setTemByLongClick(type) {
      this.setStatus({ isLongClick: true });
      if (this.circleControlAble || this.GoOut || this.LowDeHumi) {
        this.setTemByStepLoCilck(type);
      }
    },
    getStateNameById(identifier) {
      const map = this.statusMap[identifier];
      if (map) {
        const statusName = map.statusName;
        const status = this.funcDefineMap[identifier];
        const name = status.statusDefine[statusName].name;
        const stateName = `${identifier}_${name}`;
        return stateName;
      }
      return false;
    },
    // 节流函数
    throttle(value, key) {
      this.sliderValueMap[key] = value;
      clearTimeout(this.timer);
      this.timer = setTimeout(() => {
        $('#slider').roundSlider(this.sliderValueMap);
        this.sliderValueMap = {};
        // 关闭更新标志位
        this.$nextTick(() => {
          this.temChange = false;
        });
      }, 20);
    },
    // 设置温度上下限
    setSliderArea() {
      this.temChange = true;
      $('#slider').roundSlider({
        min: this.temMinVal,
        max: this.temMaxVal,
        value: this.temSetVal
      });
    },
    outPutCircleVal() {
      this.$emit('circleVal', {
        int: this.circleValInt,
        dec: this.circleValDec,
        val: this.circleVal,
        unit: this.circleValUnit,
        show: Boolean(this.circleControlAble)
      });
    },
    //创建气泡元素
    getDotRect() {
      var elem = document.querySelector('.rs-handle-dot');
      let hasDiv = document.getElementsByClassName('toolTip');
      if (!hasDiv.length) {
        var toolTip = document.body.appendChild(document.createElement('div'));
        toolTip.className = 'toolTip';
        toolTip.style.display = 'none';
        elem.appendChild(toolTip);
      }
      // const { top, left, width, height } = elem.getBoundingClientRect();
    },
    //气泡样式
    addBubblesStyle() {
      var toolTip1 = document.querySelector('.toolTip');
      var icons = document.querySelector('.icons');
      if (!this.isShowToolTip) {
        toolTip1.style.display = 'none';
        icons.style.zIndex = '3';
      } else {
        toolTip1.style.display = 'block';
        icons.style.zIndex = '-1';
      }
      this.writeTem();
    },
    //气泡内温度值
    writeTem() {
      var toolTip1 = document.querySelector('.toolTip');
      var text = document.createTextNode('');
      toolTip1.appendChild(text);
      document.querySelector('.toolTip').textContent = `${this.circleVal}` + this.circleValUnit;
    }
  }
};
</script>
